/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_BBOX_H
#define X3DTK_BBOX_H

#include "X3DTypes.h"

namespace X3DTK {

/*! \brief Class declared in <b><X3DTK/kernel.h></b> providing a bounding box.
 * 
 *  The bounding box is defined by its center and its size (diagonal).
 *
 *  \ingroup base
 */

class BBox
{
public:
  /// Constructor.
  BBox();
  /// Constructor with the center and the size as parameters.
  BBox(const SFVec3f &center, const SFVec3f &size);
  /// Constructs the bounding box from a set of vertices.
  explicit BBox(const MFVec3f &coord);
  /// Copy constructor.
  BBox(const BBox &B);
  
  /// Sets the center of the bounding box.
  inline void setCenter(const SFVec3f &center) {_center = center;};
  /// Sets the Size of the bounding box.
  inline void setSize(const SFVec3f &size) {_size = size;};
  
  /// Gets the center of the bounding box.
  inline const SFVec3f &getCenter() const {return _center;};
  /// Gets the size of the bounding box.
  inline const SFVec3f &getSize() const {return _size;};
  
private:
  SFVec3f _center;
  SFVec3f _size;  
};

}

#endif
