/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_GL_POINTSET_H
#define X3DTK_GL_POINTSET_H

#include "GL_X3DGeometryNode.h"

#include <vector>

namespace X3DTK {
namespace GL {

/*! \brief Class declared in <b><X3DTK/GL/scenegraph.h></b> providing the GL implementation
 *  of X3D::PointSet.
 *
 *  \ingroup GLSceneGraph
 */
 
class PointSet : public X3DGeometryNode
{
public:
  /// Constructor.
  PointSet();
  /// Destructor.
  virtual ~PointSet();

  /// Sets the color attribute.
  void setColor(const SFBool &color);
  /// Sets the C4UB_V3F vertex array.
  void setC4UB_V3F_vertexArray(const std::vector<C4UB_V3F> &C4UB_V3F_vertexArray);
  /// Sets the V3F vertex array.  
  void setV3F_vertexArray(const MFVec3f &V3F_vertexArray);
  
  /// Gets the color attribute.
  inline const SFBool &getColor() const {return _color;};
  /// Gets the C4UB_V3F vertex array.
  inline std::vector<C4UB_V3F> &C4UB_V3F_vertexArray() {return _C4UB_V3F_vertexArray;};
  /// Sets the V3F vertex array.  
  inline MFVec3f &V3F_vertexArray() {return _V3F_vertexArray;};
  
  /// Updates the attributes of the node.
  virtual void update();
  /// Renders the node in an openGL context.
  virtual void draw() const;  

private:
  /// Color attribute.
  SFBool _color;
  /// Point set array.
  std::vector<C4UB_V3F> _C4UB_V3F_vertexArray;
  /// Point set array.
  MFVec3f _V3F_vertexArray;
};

}
}

#endif
