/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_MESH_FACESDRAWERSTATEVARIABLES_H
#define X3DTK_MESH_FACESDRAWERSTATEVARIABLES_H

#include "MESH_SceneGraphTypes.h"
#include "MESH_Mesh.h"
#include "X3D_GLBuilder.h"
#include "GL_Renderer.h"
#include "MemReleaser.h"

#include <list>
#include <vector>

namespace X3DTK {
namespace MESH {

typedef enum {REAL, IFS, CONNCOMS} FaceMode;

/*! \brief Class declared in <b><X3DTK/MESH/facesdrawer.h></b> providing the StateVariables
 *  of the MESH::TemplateFacesDrawer processor.
 *
 *  \ingroup MESH
 */

template<class MData, class VData, class EData, class FData, bool RW>
class TemplateFacesDrawerStateVariables : public StateVariables
{
public:
  /// Constructor.
  TemplateFacesDrawerStateVariables();

  /// Initializes the traversal.
  void init();
  /// Initializes the traversal.
  void initSelected();
  /// Finishes the traversal.
  void finish();

  // Pushes the transformation matrix.
  void pushMatrix(const SFMatrix34f &transformation);
  // Pops the transformation matrix.
  void popMatrix();
  // Saves current matrix state and returns index in saved vector.
  // Call storeMesh() before calling ths function
  int getCurrentMeshId();
  
  // Saves mesh and associated current matrix.
  void storeMesh(TemplateMesh<MData, VData, EData, FData, RW> *mesh);
  
  // Get i Mesh of the model.
  inline TemplateMesh<MData, VData, EData, FData, RW> *getMesh(int i) const;
  // Gets stored transformation matrix associated with a given Mesh.
  inline const SFMatrix34f &getMatrix(int i) const;

  /// Builds the GL node.
  void buildGLNode(X3D::X3DNode *node);
  /// Renders the node in OpenGL.
  void renderGLNode(X3D::X3DNode *node);
  /// Returns true if found.
  bool getNode(X3D::X3DNode *node);
  /// Clears the GL nodes.
  void clearGLNodes();
  
  /// Sets the selection mode. 
  inline void setSelectionMode(bool selectionMode);
  /// Gets the selection mode.
  inline bool getSelectionMode() const;
  
  /// Sets the drawing mode.
  void setDrawingSelected(bool drawingSelected);
  /// Gets the drawing mode.
  inline bool getDrawingSelected() const;
  
  /// Sets the rendering mode.
  void setRenderingMode(FaceMode mode);
  /// Gets the rendering mode.
  inline FaceMode getRenderingMode() const;
    
  /// Sets the selected nodes to draw. If empty, then all the graph is drawn.
  void setSelectedNodes(const MFNode &selectedNodes);
  /// Gets the selected nodes to draw. If empty, then all the graph is drawn.
  MFNode getSelectedNodes() const;
  
private: 
  std::list<SFMatrix34f> _matrixStack;
  std::vector<SFMatrix34f> _matrixVector;
  std::vector<TemplateMesh<MData, VData, EData, FData, RW> *> _meshVector;
  std::map<X3D::X3DNode *, GL::X3DNode *> _coupleMap;
  bool _selectionMode;
  bool _drawingSelected;
  FaceMode _mode;
  MFNode _selectedNodes;
};

}
}

#include "MESH_FacesDrawerStateVariables.inl"

#endif
