/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_MESH_MESHSIMPLIFIER_H
#define X3DTK_MESH_MESHSIMPLIFIER_H

#include "X3DOnePassProcessor.h"
#include "MESH_SceneGraphTypes.h"
#include "MESH_Mesh.h"
#include "MESH_Scene.h"
#include "MESH_MeshSimplifierCoreVisitor.h"
#include "MESH_MeshSimplifierStateVariables.h"
#include "DFSGraphTraversal.h"

namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool RW> class TemplateMeshSimplifierStateVariables;

/*! \brief Class declared in <b><X3DTK/MESH/meshsimplifier.h></b> providing a processor that 
 *  simplifies the MESH scene graph into a single Mesh node and single Vertex node.
 * 
 *  The precondition is to have points in the world coordinates. The translation loses
 *  informations relative to the appearance, since a model can have different 
 *  X3DTK::X3D::Appearance nodes.
 *
 *  \ingroup MESH
 */
 
template<class MData, class VData, class EData, class FData, bool RW>
class TemplateMeshSimplifier : public X3DOnePassProcessor
{
public:
  /// Constructor.
  TemplateMeshSimplifier();
  /// Destructor.
  virtual ~TemplateMeshSimplifier();
  
  /// Computes the connected components of the scene.
  TemplateMesh<MData, VData, EData, FData, RW> *build(Scene *S);
};

/// Default TemplateMeshSimplifier.
typedef TemplateMeshSimplifier<MeshData, VertexData, EdgeData, FaceData, true> MeshSimplifier;

}
}

#include "MESH_MeshSimplifier.inl"

#endif
