/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_MESH_VERTICESDRAWER_H
#define X3DTK_MESH_VERTICESDRAWER_H

#include "MESH_VerticesDrawerStateVariables.h"
#include "DFSGraphTraversal.h"
#include "MESH_VerticesDrawerCoreVisitor.h"
#include "X3DOnePassProcessor.h"
#include "MESH_SceneGraphTypes.h"
#include "MESH_MeshData.h"

namespace X3DTK {
namespace MESH {

/*! \brief Class declared in <b><X3DTK/MESH/verticesdrawer.h></b> providing a processor that 
 *  draws the vertices of the MESH scene graph.
 *
 *  The size and color of the vertices is fixed.
 *
 *  \ingroup MESH
 */
 
template<class MData, class VData, class EData, class FData, bool RW>
class TemplateVerticesDrawer : public X3DOnePassProcessor
{
public:
  /// Constructor.
  TemplateVerticesDrawer();
  /// Destructor.
  virtual ~TemplateVerticesDrawer();
  
  /// Draws the vertices of the scene graph.
  void draw(X3DNode *N, bool selection = false);  
  // Get ith Mesh of the scene.
  inline TemplateMesh<MData, VData, EData, FData, RW> *getMesh(int i) const;
  // Gets stored transformation matrix associated with a given Mesh.
  inline const SFMatrix34f &getMatrix(int i) const;
};

/// Default TemplateVerticesDrawer.
typedef TemplateVerticesDrawer<MeshData, VertexData, EdgeData, FaceData, true> VerticesDrawer;

}
}

#include "MESH_VerticesDrawer.inl"

#endif

