/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_MESH_X3DBUILDERGLOBALVARIABLES_H
#define X3DTK_MESH_X3DBUILDERGLOBALVARIABLES_H

#include "StateVariables.h"
#include "MESH_X3DNode.h"
#include "X3D_X3DNode.h"
#include "Cloner.h"

#include <map>

namespace X3DTK {
namespace MESH {

/*! \brief Class declared in <b><X3DTK/MESH/x3dbuilder.h></b> providing the StateVariables
 *  of the MESH::TemplateX3DBuilder processor.
 *
 *  \ingroup MESH
 */

template<class MData, class VData, class EData, class FData, bool RW>
class TemplateX3DBuilderStateVariables : public StateVariables
{
public:
  /// Constructor.
  TemplateX3DBuilderStateVariables();
  /// Destructor.
  virtual ~TemplateX3DBuilderStateVariables();
  
  /// Initializes the traversal.
  virtual void init();
  /// Finishes the traversal.
  virtual void finish();
  
  /// Adds the couple of nodes.
  void addCoupleNode(SFNode N, X3D::X3DNode *NN);
  /// Pushes the node.
  void pushNode(X3D::X3DNode *N);
  /// Pops the node.
  void popNode();
    
  /// Gets the root.
  inline X3D::X3DNode *getRoot() const;
  /// Gets the top of the stack.
  inline X3D::X3DNode *getTop() const;
  /// Gets the X3D::X3DNode *related with N. Returns 0 if N not found.
  X3D::X3DNode *getNode(SFNode N) const;
  /// Clones the node.
  SFNode clone(SFNode N);
    
private:
  X3D::X3DNode *_root;
  std::list<X3D::X3DNode *> _nodeStack;
  std::map<SFNode, X3D::X3DNode *> _nodeCoupleMap;
  Cloner *_cloner;
};

}
}

#include "MESH_X3DBuilderStateVariables.inl"

#endif
