/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_CONTOURPOLYLINE2D
#define X3DTK_X3D_CONTOURPOLYLINE2D

#include "X3DTypes.h"
#include "X3D_X3DParametricGeometryNode.h"

namespace X3DTK {
namespace X3D {

/*! \brief Base class declared in <b><X3DTK/X3D/scenegraph.h></b>, part of the <a href="../../doc/X3DSpecs/Part01/components/nurbs.html">
 *  NURBS component</a> defining a ContourPolyline2D
 *  (<a href="../../doc/X3DSpecs/Part01/components/nurbs.html#ContourPolyline2D">
 *  documentation</a>).
 *
 *  \ingroup X3DSceneGraph
 */

class ContourPolyline2D : public X3DParametricGeometryNode
{
public:
  /// Constructor.
  ContourPolyline2D();
  /// Destructor.
  virtual ~ContourPolyline2D();
  
  /// Sets the point attribute.
  void setPoint(const MFVec2f &point);
  
  /// Gets the point attribute.
  inline const MFVec2f &getPoint() const {return _point;};

private:
  /// Piecewise linear curve segment.
  MFVec2f _point;
};

}
}

#endif
