/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_INLINELOADERSTATEVARIABLES_H
#define X3DTK_X3D_INLINELOADERSTATEVARIABLES_H

#include "StateVariables.h"
#include "X3D_X3DNode.h"

#include <list>

namespace X3DTK {
namespace X3D {

/*! \brief Class declared in <b><X3DTK/X3D/inlineloader.h></b> providing StateVariables
 *  for the X3D::InlineLoader processor.
 *
 *  \ingroup X3D
 */

class InlineLoaderStateVariables : public StateVariables
{
public:
  /// Constructor.
  InlineLoaderStateVariables();
  /// Destructor.
  virtual ~InlineLoaderStateVariables();
  /// Initializes the traversal.
  virtual void init();
  /// Finishes the traversal.
  virtual void finish();

  /// Adds the couple of nodes.
  void addVisitedNode(SFNode N);
  /// Pushes the node.
  void pushNode(SFNode N);
  /// Pops the node.
  void popNode();
  /// Sets the force attribute.
  void setForce(bool force);
  /// Sets the path.
  void setPath(const SFString &path);

  /// Returns TRUE if the tree had Inline nodes.
  inline bool hasInline() const {return _hasInline;};
  /// Gets the top of the stack.
  inline SFNode getTop() const {return (!_nodeStack.empty() ? _nodeStack.front() : 0);};
  /// Returns TRUE if is in force mode.
  inline bool isForced() const {return _force;};
  /// Gets the path.
  inline SFString getPath() const {return _path;};
  
private:
  bool _hasInline;
  bool _force;
  MFNode _nodeStack;
  std::list<SFNode> _visitedNodeList;
  SFString _path;
};

}
}

#endif
