/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_MESHBUILDERGROUPINGVISITOR_H
#define X3DTK_X3D_MESHBUILDERGROUPINGVISITOR_H

#include "X3D_GroupingVisitor.h"
#include "X3D_MeshBuilderStateVariables.h"
#include "X3D_X3DGroupingNode.h"
#include "X3D_StaticGroup.h"
#include "X3D_Transform.h"
#include "X3D_Switch.h"
#include "MESH_Transform.h"
#include "MESH_Switch.h"

namespace X3DTK {
namespace X3D {

class X3DChildNode;
class X3DGroupingNode;
class StaticGroup;
class Transform;
class Switch;

/*! \brief Class declared in <b><X3DTK/X3D/meshbuilder.h></b> providing a visitor
 *  of the Grouping component for the X3D::TemplateMeshBuilder processor.
 *
 *  \ingroup X3D
 */
 
template<class MData, class VData, class EData, class FData, bool RW>
class TemplateMeshBuilderGroupingVisitor : public GroupingVisitor
{
public:
  /// Constructor.
  TemplateMeshBuilderGroupingVisitor();
  
  /// Enters an X3DChildNode.
  static void enterX3DChildNode(X3DChildNode *N);
  /// Enters a Group.
  static void enterX3DGroupingNode(X3DGroupingNode *N);
  /// Enters a StaticGroup.
  static void enterStaticGroup(StaticGroup *S);
  /// Enters a Transform.
  static void enterTransform(Transform *T);
  /// Enters a Switch.
  static void enterSwitch(Switch *S);
  /// Leaves an X3DChildNode.
  static void leaveX3DChildNode(X3DChildNode *N);
  /// Leaves an X3DGroupingNode.
  static void leaveX3DGroupingNode(X3DGroupingNode *N);
};

}
}

#include "X3D_MeshBuilderGroupingVisitor.inl"

#endif
