/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_SPOTLIGHT
#define X3DTK_X3D_SPOTLIGHT

#include "X3DTypes.h"
#include "X3D_X3DLightNode.h"

namespace X3DTK {
namespace X3D {

/*! \brief Base class declared in <b><X3DTK/X3D/scenegraph.h></b>, part of the <a href="../../doc/X3DSpecs/Part01/components/lighting.html">
 *  Lighting component</a> defining a SpotLight
 *  (<a href="../../doc/X3DSpecs/Part01/components/lighting.html#SpotLight">
 *  documentation</a>).
 *
 *  \ingroup X3DSceneGraph
 */

class SpotLight : public X3DLightNode
{
public:
  /// Constructor.
  SpotLight();
  /// Destructor.
  virtual ~SpotLight();
  
  /// Sets the attenuation attribute.
  void setAttenuation(const SFVec3f &attenuation);
  /// Sets the beamWidth attribute.
  void setBeamWidth(const SFFloat &beamWidth);
  /// Sets the cutOffAngle attribute.
  void setCutOffAngle(const SFFloat &cutOffAngle);
  /// Sets the direction attribute.
  void setDirection(const SFVec3f &direction);
  /// Sets the location attribute.
  void setLocation(const SFVec3f &location);
  /// Sets the radius attribute.
  void setRadius(const SFFloat &radius);

  /// Gets the attenuation attribute.
  inline const SFVec3f &getAttenuation() const {return _attenuation;};
  /// Gets the beamWidth attribute.
  inline const SFFloat &getBeamWidth() const {return _beamWidth;};
  /// Gets the cutOffAngle attribute.
  inline const SFFloat &getCutOffAngle() const {return _cutOffAngle;};
  /// Gets the direction attribute.
  inline const SFVec3f &getDirection() const {return _direction;};
  /// Gets the location attribute.
  inline const SFVec3f &getLocation() const {return _location;};
  /// Gets the radius attribute.
  inline const SFFloat &getRadius() const {return _radius;};

private:
  /// Attenuation of the point light.
  SFVec3f _attenuation;
  /// Inner solid angle in which the light source emits light at uniform full intensity.
  SFFloat _beamWidth;
  /// Outer bound of the solid angle.
  SFFloat _cutOffAngle;
  /// Direction vector of the illumination emanating from the light source.
  SFVec3f _direction;
  /// Translation offset of the centre point of the light.
  SFVec3f _location;
  /// Maximal distance from location that may be illuminated by the light source.
  SFFloat _radius;
};

}
}

#endif
