/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_TRIMMEDSURFACE_H
#define X3DTK_X3D_TRIMMEDSURFACE_H

#include "X3DTypes.h"
#include "X3D_X3DParametricGeometryNode.h"

namespace X3DTK {
namespace X3D {

class Contour2D;
class NurbsSurface;

/*! \brief Base class declared in <b><X3DTK/X3D/scenegraph.h></b>, part of the <a href="../../doc/X3DSpecs/Part01/components/nurbs.html">
 *  NURBS component</a> defining a TrimmedSurface
 *  (<a href="../../doc/X3DSpecs/Part01/components/nurbs.html#TrimmedSurface">
 *  documentation</a>).
 *
 *  \ingroup X3DSceneGraph
 */

class TrimmedSurface : public X3DParametricGeometryNode
{
public:
  /// Constructor.
  TrimmedSurface();
  /// Destructor.
  virtual ~TrimmedSurface();
  
  /// Gets the TrimmingContour attribute.
  inline const MFNode &getTrimmingContour() const {return _trimmingContour;};
  /// Gets the Surface attribute.
  inline NurbsSurface *getSurface() const {return _surface;};

private:
  /// Set of Contour2D nodes.
  MFNode _trimmingContour;
  /// NurbsSurface node to be trimmed.
  NurbsSurface *_surface;
};

}
}

#endif
