/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_X3DTEXTURECOORDINATENODE_H
#define X3DTK_X3D_X3DTEXTURECOORDINATENODE_H

#include "X3DTypes.h"
#include "X3D_X3DGeometricPropertyNode.h"

namespace X3DTK {
namespace X3D {

/* Base class declared in <b><X3DTK/X3D/scenegraph.h></b>, part of the <a href="../../doc/X3DSpecs/Part01/components/texturing.html">
 *  Texturing component</a> defining an X3DTextureCoordinateNode
 *  (<a href="../../doc/X3DSpecs/Part01/components/texturing.html#X3DTextureCoordinateNode">
 *  documentation</a>).
 *
 *  \ingroup X3DSceneGraph
 */

class X3DTextureCoordinateNode : public X3DGeometricPropertyNode
{
public:  
  /// Constructor.
  X3DTextureCoordinateNode();
  /// Destructor.
  virtual ~X3DTextureCoordinateNode();
  
  /// Sets the Point attribute.
  void setPoint(const MFVec2f &point);
  
  /// Gets the Point attribute.
  inline MFVec2f &getPoint() {return _point;};  
  /// Gets the Point attribute.
  inline const MFVec2f &getPoint() const {return _point;};

private:  
  /// The point array.
  MFVec2f _point;  
};

}
}

#endif
